import 'package:flutter/material.dart';
import 'package:flutter_chessboard_js/flutter_chessboard_js.dart';

/// Panel to display notation
class ChessboardNotation extends StatefulWidget {
  ChessboardNotation({
    required this.flipBoard,
    required this.height,
    required this.width,
    this.backgroundColor = Colors.black,
    required this.controller,
    required this.notationTextColor,
    this.notationTopBar,
//    this.onTapMove,
  });

  final VoidCallback flipBoard;
  final double height;
  final double width;
  final Color backgroundColor;
  final ChessboardController controller;
  final Color notationTextColor;
  final Widget? notationTopBar;
//  final Function(int)? onTapMove;

  @override
  _ChessboardNotationState createState() => _ChessboardNotationState();
}

class _ChessboardNotationState extends State<ChessboardNotation> {
  final scrollerController = ScrollController();
  int? selectedMove;
  late Color notationTileSelectedColor;

  @override
  void initState() {
    notationTileSelectedColor = Colors.red; //widget.color.withOpacity(0.4);
    widget.controller.addListener(() {
      if (mounted)
        setState(() {
          if (scrollerController.hasClients) {
            /// animate to the end of the scrollable list with each move
            scrollerController.animateTo(
              scrollerController.position.maxScrollExtent,
              duration: Duration(milliseconds: 600),
              curve: Curves.fastOutSlowIn,
            );
          }
        });
    });
    super.initState();
  }

  @override
  void didUpdateWidget(covariant ChessboardNotation oldWidget) {
    if (oldWidget.controller != this.widget.controller) {
      widget.controller.addListener(() {
        if (mounted)
          setState(() {
            print('settings notationTileSelected');
            selectedMove = widget.controller.getSanHistory().length - 1;
          });
      });
    }
    super.didUpdateWidget(oldWidget);
  }

  @override
  void dispose() {
    scrollerController.removeListener(() {});
    widget.controller.removeListener(() {});
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    final moves = widget.controller.gameLogic.getHistorySAN();

    return Container(
      width: widget.width,
      height: widget.height,
      color: widget.backgroundColor,
      child: Padding(
        padding: const EdgeInsets.all(8.0),
        child: Column(
          mainAxisSize: MainAxisSize.max,
          children: [
            if (widget.notationTopBar != null) widget.notationTopBar!,
            if (moves.isNotEmpty)
              Expanded(
                child: SingleChildScrollView(
                  controller: scrollerController,
                  child: Column(
                    mainAxisSize: MainAxisSize.max,
                    children: [
                      ...buildNotationTile(moves),
                    ],
                  ),
                ),
              ),
            if (moves.isEmpty) Spacer(),
            Row(
              children: [
                IconButton(
                    onPressed: widget.flipBoard,
                    icon: Icon(
                      Icons.flip_camera_android,
                      color: widget.notationTextColor,
                    )),

                /// testing enable/disable user moves
//                IconButton(
//                    onPressed: () => widget.controller.toggleEnableMove(),
//                    icon: Icon(
//                      widget.controller.enableUserMoves
//                          ? Icons.stop
//                          : Icons.play_arrow_rounded,
//                      color: widget.notationTextColor,
//                    )),
              ],
            ),
          ],
        ),
      ),
    );
  }

//  Color setTileColor(int tileIndex) {
//    print('setting color');
//    if (tileIndex == notationTileSelected)
//      return notationTileSelectedColor;
//    else
//      return Colors.transparent;
//  }

  List<Widget> buildNotationTile(List<String> moves) {
    List<Widget> tiles = [];

    int count = 1;
    for (int i = 0; i < moves.length; i = i + 2) {
      final temp = Row(
        children: [
          Container(
              decoration: BoxDecoration(
                  border: Border(
                      right: BorderSide(color: widget.notationTextColor))),
              height: 30,
              width: 30,
              child: Center(
                  child: Text('$count',
                      style: TextStyle(color: widget.notationTextColor)))),
          Expanded(
            flex: 2,
            child: NotationTile(
              color: widget.notationTextColor,
              tileBackgroundColor: widget.backgroundColor,
              moveNumber: i,
              move: moves[i],
//              onTap: widget.onTapMove,
            ),
          ),
          Expanded(
            flex: 2,
            child: (i + 1 < moves.length)
                ? NotationTile(
                    color: widget.notationTextColor,
                    tileBackgroundColor: widget.backgroundColor,
                    move: moves[i + 1],
                    moveNumber: i + 1,
//                    onTap: widget.onTapMove,
                  )
                : const SizedBox(),
          ),
        ],
      );
      tiles.add(temp);
      count++;
    }
    tiles.add(const SizedBox(height: 20));
    return tiles;
  }
}

class NotationTile extends StatefulWidget {
  const NotationTile({
    Key? key,
    required this.move,
    required this.moveNumber,
    required this.color,
//    required this.onTap,
    this.tileBackgroundColor = Colors.transparent,
  }) : super(key: key);

  final String move;
  final int moveNumber;
  final Color color;
  final Color tileBackgroundColor;
//  final Function(int)? onTap;

  @override
  _NotationTileState createState() => _NotationTileState();
}

class _NotationTileState extends State<NotationTile> {
  late Color backgroundColor;

  @override
  void initState() {
    backgroundColor = widget.tileBackgroundColor;
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      height: 30,
      color: backgroundColor,

      /// TODO this is for when selecting a move in the notation works
//      child: MouseRegion(
//        onEnter: (details) {
//          setState(() {
//            backgroundColor = Theme.of(context).hoverColor;
//          });
//        },
//        onExit: (details) {
//          setState(() {
//            backgroundColor = widget.tileBackgroundColor;
//          });
//        },
//        child: GestureDetector(
//          onTap: widget.onTap != null ? widget.onTap!(widget.moveNumber) : null,
      child: Center(
          child: Text(widget.move, style: TextStyle(color: widget.color))),
    )
//      ),
//    )
        ;
  }
}
