import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:flutter_chessboard_js/flutter_chessboard_js.dart';
import 'package:chessjs/chessjs.dart' as chessjs;
import '../chess_vectors/chess_vectors_flutter.dart';

/// Show dialog when pawn reaches last square
Future<String?> promotionDialog(
    BuildContext context, ChessboardController controller) async {
  return showDialog<String>(
    context: context,
    barrierDismissible: false,
    builder: (BuildContext context) {
      final color = controller.gameLogic.turn;
      return AlertDialog(
        title: Text('Choose promotion'),
        content: Row(
          mainAxisAlignment: MainAxisAlignment.spaceEvenly,
          children: <Widget>[
            InkWell(
              child: color == chessjs.WHITE ? WhiteQueen() : BlackQueen(),
              onTap: () {
                Navigator.of(context).pop("q");
              },
            ),
            InkWell(
              child: color == chessjs.WHITE ? WhiteRook() : BlackRook(),
              onTap: () {
                Navigator.of(context).pop("r");
              },
            ),
            InkWell(
              child: color == chessjs.WHITE ? WhiteBishop() : BlackBishop(),
              onTap: () {
                Navigator.of(context).pop("b");
              },
            ),
            InkWell(
              child: color == chessjs.WHITE ? WhiteKnight() : BlackKnight(),
              onTap: () {
                Navigator.of(context).pop("n");
              },
            ),
          ],
        ),
      );
    },
  ).then((value) {
    return value;
  });
}

/// Get image to display on square
Widget getImageToDisplay(
    {required double size,
    required ChessboardController controller,
    required squareName}) {
  Widget imageToDisplay = const SizedBox();

  if (controller.gameLogic.get(squareName) == null) {
    const SizedBox();
  }

  String piece = (controller.gameLogic.get(squareName)!.color == chessjs.WHITE
          ? 'W'
          : 'B') +
      controller.gameLogic.get(squareName)!.type.toUpperCase();

  switch (piece) {
    case "WP":
      imageToDisplay = WhitePawn(size: size);
      break;
    case "WR":
      imageToDisplay = WhiteRook(size: size);
      break;
    case "WN":
      imageToDisplay = WhiteKnight(size: size);
      break;
    case "WB":
      imageToDisplay = WhiteBishop(size: size);
      break;
    case "WQ":
      imageToDisplay = WhiteQueen(size: size);
      break;
    case "WK":
      imageToDisplay = WhiteKing(size: size);
      break;
    case "BP":
      imageToDisplay = BlackPawn(size: size);
      break;
    case "BR":
      imageToDisplay = BlackRook(size: size);
      break;
    case "BN":
      imageToDisplay = BlackKnight(size: size);
      break;
    case "BB":
      imageToDisplay = BlackBishop(size: size);
      break;
    case "BQ":
      imageToDisplay = BlackQueen(size: size);
      break;
    case "BK":
      imageToDisplay = BlackKing(size: size);
      break;
    default:
      imageToDisplay = const SizedBox();
  }

  return imageToDisplay;
}

/// function to check if character is adjacent to another character
bool isAdjacentFile(String columnName, String isAdjacent) {
  return (columnName.contains(
          String.fromCharCode((isAdjacent[0][0]).codeUnitAt(0) - 1)) ||
      columnName
          .contains(String.fromCharCode((isAdjacent[0][0]).codeUnitAt(0) + 1)));
}
